#pragma once

#include <UIMLView.h>

#include "NetworkAPIProcessor.h"

class QuestGraphElement : public Framework::UIMLElement
{
public:
    QuestGraphElement();
    //! prft, ob dieses UIML Element fr ein bestimmtes xml Element zustndig
    //! ist
    bool isApplicableFor(Framework::XML::Element& element) override;
    //! erstellt eine neue Zeichnung zu einem gegebenen xml Element
    Framework::Zeichnung* parseElement(Framework::XML::Element& element,
        Framework::UIMLContainer& generalFactory) override;
    bool updateElement(Framework::XML::Element& element,
        Framework::Zeichnung& z,
        Framework::UIMLContainer& generalFactory) override;
    //! wendet die layout parameter zu einer Zeichnung an
    void layout(Framework::XML::Element& element,
        Framework::Zeichnung& z,
        int pWidth,
        int pHeight,
        Framework::UIMLContainer& generalLayouter) override;
};

class QuestGraphItemElement : public Framework::UIMLElement
{
public:
    QuestGraphItemElement();
    //! prft, ob dieses UIML Element fr ein bestimmtes xml Element zustndig
    //! ist
    bool isApplicableFor(Framework::XML::Element& element) override;
    //! erstellt eine neue Zeichnung zu einem gegebenen xml Element
    Framework::Zeichnung* parseElement(Framework::XML::Element& element,
        Framework::UIMLContainer& generalFactory) override;
    bool updateElement(Framework::XML::Element& element,
        Framework::Zeichnung& z,
        Framework::UIMLContainer& generalFactory) override;
    //! wendet die layout parameter zu einer Zeichnung an
    void layout(Framework::XML::Element& element,
        Framework::Zeichnung& z,
        int pWidth,
        int pHeight,
        Framework::UIMLContainer& generalLayouter) override;
};

class QuestGraphItem : public Framework::ZeichnungHintergrund
{
private:
    int layerIndex;
    int nodeIndex;
    Framework::Array<QuestGraphItem*> previousLayersConnections;
    Framework::Array<QuestGraphItem*> nextLayersConnections;
    Framework::Text id;
    Framework::Text requirements;
    Framework::Text onClick;
    bool finished;
    bool mainQuest;
    bool virtualNode;

public:
    QuestGraphItem();

    void render(Framework::Bild& rObj) override;
    void doMausEreignis(Framework::MausEreignis& me, bool userRet) override;
    const Framework::Text& getId() const;
    const Framework::Text& getRequirements() const;

    bool calculateLaxerIndex();
    int getLayerIndex() const;
    void setVirtual(bool virtualNode);
    void setNodeIndex(int index);

    void addNextLayerRef(QuestGraphItem* zItem);
    void addPreviousLayerRef(QuestGraphItem* zItem);

    const Framework::Array<QuestGraphItem*>& getNextLayersConnections() const;
    int getNodeIndex() const;
    bool isMainQuest() const;

    friend QuestGraphItemElement;
};

class QuestGraphItemLayer : public Framework::ReferenceCounter
{
private:
    Framework::RCArray<QuestGraphItem> items;

public:
    QuestGraphItemLayer();

    bool tick(double tickVal);
    void render(Framework::Bild& rObj);
    void doMausEreignis(Framework::MausEreignis& me);

    void addItem(QuestGraphItem* item);

    bool sortItems();
    bool fixItemPositions();
    const Framework::RCArray<QuestGraphItem>& getItems() const;
    int getLeyerHeight() const;
    void centerVertically(int pos);
};

class QuestGraph : public Framework::ZeichnungHintergrund
{
private:
    Framework::Text id;
    Framework::RCArray<QuestGraphItemLayer> layers;
    QuestGraphItemLayer invalidNodes;
    Framework::Text collectionName;

public:
    QuestGraph();

    void render(Framework::Bild& rObj) override;
    void doMausEreignis(Framework::MausEreignis& me, bool userRet) override;

    void addVirtualConnectionNodes();
    void sortItems();
    void fixItemPositions();
    void addItem(QuestGraphItem* item);

    friend QuestGraphElement;
};